@@ -6,11 +6,21 @@ use utf8;
 use Exporter 5.57 'import';
 use Unicode::UCD qw(charscript);
 
-our $VERSION = '0.06';
-our @EXPORT_OK = qw(sentence_iterator phrase_iterator extract_presuf extract_words tokenize_by_script);
+our $VERSION = '0.07';
+our @EXPORT_OK = qw(sentence_iterator phrase_iterator presuf_iterator word_iterator extract_presuf extract_words tokenize_by_script);
 
 use List::Util qw(uniq pairmap);
 
+sub exhaust {
+    my ($iter, $cb) = @_;
+    my @list;
+    while(defined(my $x = $iter->())) {
+        push @list, $x;
+        $cb->($x) if defined($cb);
+    }
+    return @list;
+}
+
 sub grep_iterator {
     my ($iter, $cb) = @_;
     return sub {
@@ -51,62 +61,83 @@ sub sentence_iterator {
                            )/gx);
         }
         return shift @sentences;
-    }    
+    }
 }
 
-sub extract_presuf {
-    my ($input_iter, $output_cb, $opts) = @_;
+sub presuf_iterator {
+    my ($input_iter, $opts) = @_;
 
     my %stats;
-    my %extracted;
     my $threshold = $opts->{threshold} || 9; # an arbitrary choice.
     my $lengths   = $opts->{lengths} || [2,3];
-    my $text;
 
     my $phrase_iter = grep_iterator(
         phrase_iterator( $input_iter ),
         sub { /\A\p{Han}+\z/ }
     );
-    while (my $phrase = $phrase_iter->()) {
-        for my $len ( @$lengths ) {
-            my $re = '\p{Han}{' . $len . '}';
-            next unless length($phrase) >= $len * 2 && $phrase =~ /\A($re) .* ($re)\z/x;
-            my ($prefix, $suffix) = ($1, $2);
-            $stats{prefix}{$prefix}++ unless $extracted{$prefix};
-            $stats{suffix}{$suffix}++ unless $extracted{$suffix};
 
-            for my $x ($prefix, $suffix) {
-                if (! $extracted{$x}
-                    && $stats{prefix}{$x}
-                    && $stats{suffix}{$x}
-                    && $stats{prefix}{$x} > $threshold
-                    && $stats{suffix}{$x} > $threshold
-                ) {
-                    $extracted{$x} = 1;
-                    delete $stats{prefix}{$x};
-                    delete $stats{suffix}{$x};
+    my (%extracted, @extracted);
+    return sub {
+        if (@extracted) {
+            return shift @extracted;
+        }
 
-                    $output_cb->($x, \%extracted);
+        while (!@extracted && defined(my $phrase = $phrase_iter->())) {
+            for my $len ( @$lengths ) {
+                my $re = '\p{Han}{' . $len . '}';
+                next unless length($phrase) >= $len * 2 && $phrase =~ /\A($re) .* ($re)\z/x;
+                my ($prefix, $suffix) = ($1, $2);
+                $stats{prefix}{$prefix}++ unless $extracted{$prefix};
+                $stats{suffix}{$suffix}++ unless $extracted{$suffix};
+
+                for my $x ($prefix, $suffix) {
+                    if (! $extracted{$x}
+                        && $stats{prefix}{$x}
+                        && $stats{suffix}{$x}
+                        && $stats{prefix}{$x} > $threshold
+                        && $stats{suffix}{$x} > $threshold
+                    ) {
+                        $extracted{$x} = 1;
+                        delete $stats{prefix}{$x};
+                        delete $stats{suffix}{$x};
+
+                        push @extracted, $x;
+                    }
                 }
             }
         }
-    }
 
+        if (@extracted) {
+            return shift @extracted;
+        }
 
-    return \%extracted;
+        return undef;
+    };
 }
 
-sub extract_words {
+sub extract_presuf {
+    my ($input_iter, $opts) = @_;
+    return [ exhaust(presuf_iterator($input_iter, $opts)) ];
+}
+
+sub word_iterator {
     my ($input_iter) = @_;
 
-    my (%lcontext, %rcontext);
+    my $threshold = 5;
+    my (%lcontext, %rcontext, %word, @words);
 
-    while( my $txt = $input_iter->() ) {
-        my @phrase = split /\P{Letter}/, $txt;
-        for (@phrase) {
-            next unless /\A\p{Han}+\z/;
+    my $phrase_iter = grep_iterator(
+        phrase_iterator( $input_iter ),
+        sub { /\A\p{Han}+\z/ }
+    );
 
-            my @c = split("", $_);
+    return sub {
+        if (@words) {
+            return shift @words;
+        }
+
+        while (!@words && defined( my $txt = $phrase_iter->() )) {
+            my @c = split("", $txt);
 
             for my $i (0..$#c) {
                 if ($i > 0) {
@@ -114,8 +145,15 @@ sub extract_words {
                     for my $n (2,3) {
                         if ($i >= $n) {
                             my $tok = join('', @c[ ($i-$n+1) .. $i] );
-                            if (length($tok) > 1) {
-                                $lcontext{ $tok }{$c[$i - $n]}++;
+                            unless ($word{$tok}) {
+                                if (length($tok) > 1) {
+                                    $lcontext{ $tok }{$c[$i - $n]}++;
+                                }
+
+                                if ($threshold <= (keys %{$lcontext{$tok}}) && $threshold <= (keys %{$rcontext{$tok}})) {
+                                    $word{$tok} = 1;
+                                    push @words, $tok;
+                                }
                             }
                         }
                     }
@@ -125,25 +163,27 @@ sub extract_words {
                     for my $n (2,3) {
                         if ($i + $n <= $#c) {
                             my $tok = join('', @c[$i .. ($i+$n-1)]);
-                            if (length($tok) > 1) {
-                                $rcontext{ $tok }{ $c[$i+$n] }++;
+                            unless ($word{$tok}) {
+                                if (length($tok) > 1) {
+                                    $rcontext{ $tok }{ $c[$i+$n] }++;
+                                }
+
+                                if ($threshold <= (keys %{$lcontext{$tok}}) && $threshold <= (keys %{$rcontext{$tok}})) {
+                                    $word{$tok} = 1;
+                                    push @words, $tok;
+                                }
                             }
                         }
                     }
                 }
             }
         }
+        return shift @words;
     }
+}
 
-    my @tokens = uniq((keys %lcontext), (keys %rcontext));
-    my @words;
-    my $threshold = 5;
-    for my $x (@tokens) {
-        next unless ($threshold <= (keys %{$lcontext{$x}}) && $threshold <= (keys %{$rcontext{$x}}));
-        push @words, $x;
-    }
-
-    return \@words;
+sub extract_words {
+    return [ exhaust(word_iterator(@_)) ];
 }
 
 sub tokenize_by_script {
@@ -203,16 +243,19 @@ trivial example to open a file as an input iterator:
             return decode_utf8($line);
         }
     }
-    
+
     my $input_iter = open_as_iterator("/data/corpus.txt");
 
 This C<$input_iter> can be then passed as arguments to different subroutines.
 
+Although in the rest of this document, `Iter` is used as a Type
+notation for iterators. It is the same as a CODE reference.
+
 =head1 EXPORTED SUBROUTINES
 
 =over 4
 
-=item extract_words( $input_iter ) #=> ArrayRef[Str]
+=item word_iterator( $input_iter ) #=> Iter
 
 This extracts words from Chinese text. A word in Chinese text is a token
 with N charaters. These N characters is often used together in the input and
@@ -222,6 +265,21 @@ The input parameter is a iterator -- a subroutine that must return a string of
 Chinese text each time it is invoked. Or, when the input is exhausted, it must
 return undef. For example:
 
+    open my $fh, '<', 'book.txt';
+    my $word_iter = word_iterator(
+        sub {
+            my $x = <$fh>;
+            return decode_utf8 $x;
+        });
+
+The type of return value is Iter (CODE ref).
+
+=item extract_words( $input_iter ) #=> ArrayRef[Str]
+
+This does the same thing as C<word_iterator>, but retruns the exhausted list instead of iterator.
+
+For example:
+
     open my $fh, '<', 'book.txt';
     my $words = extract_words(
         sub {
@@ -235,45 +293,28 @@ It is likely that this subroutine returns an empty ArrayRef with no contents.
 It is only useful when the volume of input is a leats a few thousands of
 characters. The more, the better.
 
-=item extract_presuf( $input_iter, $output_cb, $opts ) #=> HashRef
+=item presuf_iterator( $input_iter, $opts) #=> Iter
 
 This subroutine extract meaningful tokens that are prefix or suffix of
-input. Comparing to C<extract_word>, it yields extracted tokens frequently
-by calling C<$output_cb>.
+input.
 
-It is used like this:
-
-    my $extracted = extract_presuf(
-        \&next_input,
-        sub {
-            my ($token, $extracted) = @_;
-
-            ...
-        },
-        +{
-            threshold => 9,
-            lengths => [ 2,3 ],
-        }
-    );
-
-The C<$output_cb> callback is passed two arguments. The first one is the new
-C<$token> that appears more then C<$threshold> times as a prefix and as a
-suffix. The second arguments is a HashRef with keys being the set of all
-extracted tokens. The very same HashRef is also going to be the return value
-of this subroutine.
-
-The 3rd argument is a HashRef with parameters to the internal algorithm.
-C<threshold> should be an Int, C<lengths> should be an ArrayRef[Int] and
-that constraints the lengths of prefixes and suffixes to be extracted.
+The 2nd argument C<$opts> is a HashRef with parameters C<threshold>
+and C<lengths>. C<threshold> should be an Int, C<lengths> should be an
+ArrayRef[Int] and that constraints the lengths of prefixes and
+suffixes to be extracted.
 
 The default value for C<threshold> is 9, while the default value for C<lengths> is C<[2,3]>
 
-=item sentences_iterator( $input_iter ) #=> CodeRef
+=item extract_presuf( $input_iter, $opts ) #=> ArrayRef[Str]
+
+Similar to C<presuf_iterator>, but returns a ArrayRef[Str] instead.
+
+=item sentence_iterator( $input_iter ) #=> Iter
 
 This subroutine split input into sentences. It takes an text iterator,
 and returns another one.
 
-=item phrase_iterator( $input_iter ) #=> CodeRef
+=item phrase_iterator( $input_iter ) #=> Iter
 
 This subroutine split input into smallelr phrases. It takes an text iterator,
 and returns another one.
